//#define LOGGER

#property icon "\\Images\\ovo128.ico";
#property version   "103.002"
#property copyright "Ovo (c) 2015"
#property link      "http://ovo.cz/#"
#property indicator_chart_window
#property description "Synchronized scrolling of multiple charts."
#property description "Attaching indicator to multiple charts will bind their scrolling. Drag one chart and others follow. Scroll it by mouse and keyboard. Other charts will follow."
#property description "May be used with any timeframes and charts, including mixed symbols and offline charts."

#define CHART_EVENT_ENABLE
#define LINE_NAME "SYNCSCROLLLINE"

#include <Ovo/MT4IndicatorManager.class.mqh>

input int CHANNEL = 0; // Chart group, which scroll together
input bool SHOW_LINE = true; // Show sync line

bool MASTER = false;


void main() {
   manager.add(new ChartMoveListener());
   manager.add(new SyncRequestListener());
   manager.add(new MasterSwitchListener());
   LogoPrinter* logo = manager.add(new LogoPrinter());
   logo.run();
   ChartSetInteger(0,CHART_EVENT_MOUSE_MOVE, true);
}

void showVLine(int focusedIndex, datetime altTime = 0) {
   TRACEIN_;
   if (SHOW_LINE) {
      datetime focusedDate = (datetime)INT_MAX;
      double focusedPrice = 0.0;
      
      if (focusedIndex >= 0) {
         focusedDate = Time[focusedIndex];
         focusedPrice = High[focusedIndex];
      }
      
      if (ObjectCreate(LINE_NAME, OBJ_VLINE, 0, 0, 0)) {
         ObjectSetInteger(0, LINE_NAME, OBJPROP_BACK, true);
      }
      color lineColour = (color)ChartGetInteger(0, CHART_COLOR_GRID);
      ObjectSet(LINE_NAME, OBJPROP_COLOR, lineColour);
      ObjectSet(LINE_NAME, OBJPROP_TIME1,focusedDate);
      ObjectSet(LINE_NAME, OBJPROP_PRICE1,focusedPrice);
      string time = TimeToStr(altTime>0?altTime:focusedDate, TIME_MINUTES|TIME_SECONDS);
      ObjectSetText(LINE_NAME, time);
      ObjectSetString(0, LINE_NAME, OBJPROP_TOOLTIP, TimeToStr(altTime>0?altTime:focusedDate, TIME_MINUTES|TIME_SECONDS) + "\n\nFor color change use\nGrid Color in Chart Settings.");
    }
   TRACEOUT_;
}

/**
 * Listens to scroling chart by a mouse.
 */
class ChartMoveListener : public MT4ChartListener {
   /** The first visible bar from the recent call 
   */
   int recentBar;
   int focusedIndex;
   bool wasMousePressed;
public:

   ChartMoveListener():focusedIndex(-1) {}
   /**
    * Event handler.
    */
   void run	(MT4ChartEvent* event)	{
        TRACEIN_;    
      // unlock the auto scroll when mouse is down
      if (event.isMouseMove()) {
         int isMousePressed = (event.getStringValue()=="1");
         
         // unlock the chart
         if (!wasMousePressed  && isMousePressed && ChartGetInteger(0,CHART_AUTOSCROLL)) {
             ChartSetInteger(0, CHART_AUTOSCROLL, 0);             
         }
                  
         // lock the chart if behind the last bar
         if (wasMousePressed  && !isMousePressed && focusedIndex <= 0) {
            ChartSetInteger(0, CHART_AUTOSCROLL, 1);
            ObjectDelete(LINE_NAME);
         }

         wasMousePressed = isMousePressed;
      }

      // sets to master mode, if something different than "chart change" gets fired.
      if(!MASTER && !event.isChartChange()) {
         MASTER = true;
         MT4CustomEvent e;
         e.setCustomType(EVENT_SYNCMASTER_BROADCAST);
         e.setDoubleNumber(CHANNEL);
         e.broadcastToAll(true);         
      }
      
      // return if chart is not in master mode
      if (!MASTER || !(event.isChartChange() || event.isKeyDown())) {
         TRACEOUT_;
         return;
      }
            
      int firstBarIndex = (int)ChartGetInteger(0,CHART_FIRST_VISIBLE_BAR);            
      
      // first bar in the chart changed
      if (firstBarIndex>0 && recentBar != firstBarIndex) {

         // update first bar record
         recentBar = firstBarIndex;
         
         //get the bar at marker
         int barsOnChart = (int)ChartGetInteger(0,CHART_WIDTH_IN_BARS);
         double percentFromLeft = 100 - ChartGetDouble(0,CHART_SHIFT_SIZE);
         if (!ChartGetInteger(0,CHART_SHIFT)) {
            percentFromLeft = 100;
            barsOnChart -= 2;
         }

         focusedIndex = firstBarIndex - (int)round(barsOnChart * percentFromLeft/100.0);
         datetime focusedDate = (datetime)INT_MAX;
         if (focusedIndex >= 0) {
            focusedDate = Time[focusedIndex];
            
            long realVolume[];
               // the real_volume starts with 07
            if((CopyRealVolume(NULL, 0, focusedIndex, 1, realVolume)>0) && ((realVolume[0]&0xff00000000000000)==0x0700000000000000)) {
               // strip off 07
               focusedDate = (datetime)(realVolume[0]&0x00ffffffffffffff);
            }            
         }
         showVLine(focusedIndex, focusedDate);
         ChartRedraw();

         // fire central bar position to other charts         
         MT4CustomEvent e;
         e.setCustomType(EVENT_SYNCCHART_BROADCAST);
         e.setLongNumber(focusedDate);
         e.setStringValue(_Symbol);
         e.setDoubleNumber(CHANNEL);
         e.broadcastToAll(true);
      }
      
      if (event.isChartChange()) {
         //Print (_Period, " ", focusedIndex);
         ChartSetInteger(0, CHART_AUTOSCROLL, (focusedIndex <=0));
         if (focusedIndex <=0) {
            ObjectDelete(LINE_NAME);
         }
      }      
      TRACEOUT_;
   }   
};

/**
 * Synchronzes central bar to event data
 */
class  SyncRequestListener:public MT4CustomListener {
   
   long lastTime; 
   public:
   void run(MT4CustomEvent *event) {   
      TRACEIN_;
      if (!MASTER && event.getCustomType() == EVENT_SYNCCHART_BROADCAST && event.getDoubleNumber() == CHANNEL && event.getLongNumber() != lastTime) {
         TRACE("incomming event");
         lastTime = event.getLongNumber();
         int barsOnChart = (int)ChartGetInteger(0,CHART_WIDTH_IN_BARS);         
         datetime focusedDate = (datetime)event.getLongNumber();
         int focusedIndex = -1;
         if (focusedDate == INT_MAX) {            
            ChartSetInteger(0,CHART_AUTOSCROLL,1);
            ChartNavigate(0, CHART_END, -1);
            ObjectDelete(LINE_NAME);
         } else {
            focusedIndex = iBarShift(NULL, 0, focusedDate);
            if (focusedIndex==-1) { // errors after chart refresh
               TRACEOUT_;
               return;
            }
            datetime realTime = Time[focusedIndex];
            
            long realVolume[];
            if((CopyRealVolume(NULL, 0, focusedIndex, 1, realVolume)>0) && ((realVolume[0]&0xff00000000000000)==0x0700000000000000)) {
               // the real_volume starts with 07
                  // strip off 07
               realTime = (datetime)(realVolume[0]&0x00ffffffffffffff);
               // check if same date
               if (realTime < focusedDate) {
                  datetime tmpTime = realTime;
                  while (tmpTime < focusedDate) {
                     focusedIndex--;
                     //Print(realTime, "  -  ", focusedDate);
                     if(CopyRealVolume(NULL, 0, focusedIndex, 1, realVolume)>0) {
                        realTime = tmpTime;
                        tmpTime = (datetime)(realVolume[0]&0x00ffffffffffffff);
                     } else {
                        break;
                     }
                  }
                  focusedIndex++;
               }
            }
            
            // count the first bar from marker bar
            double percentFromLeft = 100 - ChartGetDouble(0,CHART_SHIFT_SIZE);
            if (!ChartGetInteger(0,CHART_SHIFT)) {
               percentFromLeft = 100;
               barsOnChart -=2;
            }
            showVLine(focusedIndex, realTime);
            TRACE("");
            int firstBarIndex = focusedIndex + (int)round(barsOnChart * percentFromLeft/100.0);
            TRACE("");
            firstBarIndex = min(firstBarIndex, Bars-1);
            TRACE("");
            ChartSetInteger(0, CHART_AUTOSCROLL, 0);
            TRACE("");
            ChartNavigate(0, CHART_BEGIN, Bars - firstBarIndex-1);
            TRACE("");
         }
      }
      TRACEOUT_;
   }
     
};

/** 
 * Switches to slave mode if some other chart gets control
 */
class MasterSwitchListener: public MT4CustomListener {
public:
   /**
    * event handler
    */
   void run(MT4CustomEvent* e) {
      TRACEIN_;
      if (e.getCustomType() == EVENT_SYNCMASTER_BROADCAST && e.getDoubleNumber() == CHANNEL) {
         MASTER = false;
      }
      TRACEOUT_;
   }      
};


class LogoPrinter:public MT4TimerRecuringListener {
int counter;
public:
static const string LABEL_OVO;
LogoPrinter():MT4TimerRecuringListener(4000) {}
~LogoPrinter() {
   ObjectDelete(LABEL_OVO);
   ObjectDelete(LINE_NAME);
}
   void run() {
      TRACEIN_;
      ObjectDelete(LABEL_OVO);
      if (counter++%8 == 0 && ObjectCreate(LABEL_OVO, OBJ_LABEL, 0, 0, 0)) {         
         ObjectSetText(LABEL_OVO,"sync channel " + IntegerToString(CHANNEL));
         ObjectSetInteger(0,LABEL_OVO, OBJPROP_FONTSIZE, 7);
         ObjectSetInteger(0,LABEL_OVO, OBJPROP_XDISTANCE, 5);
         ObjectSetInteger(0,LABEL_OVO, OBJPROP_CORNER, CORNER_RIGHT_LOWER);
         ObjectSetString(0,LABEL_OVO, OBJPROP_TOOLTIP, "Visit OVO.CZ\nfor more utilities!");
         ObjectSetInteger(0,LABEL_OVO, OBJPROP_ANCHOR, ANCHOR_RIGHT_LOWER);
         ObjectSetInteger(0,LABEL_OVO, OBJPROP_YDISTANCE, 2);
      }
      TRACEOUT_;
   }
};
static const string LogoPrinter::LABEL_OVO = "OvoLabel";